@@ -21,14 +21,23 @@
 The global things in Frescobaldi.
 """
 
-from __future__ import unicode_literals
-from __future__ import print_function
+
 
 import os
 import sys
+import platform
+import importlib.util
 
-from PyQt4.QtCore import QSettings, QThread
-from PyQt4.QtGui import QApplication
+
+from PyQt5.QtCore import QSettings, Qt, QThread
+from PyQt5.QtWidgets import QApplication, QMenuBar
+
+### needed for QWebEngine
+### it wants those two things be done before constructing QApplication()
+if importlib.util.find_spec('PyQt5.QtWebEngineWidgets'):
+    import PyQt5.QtWebEngineWidgets
+    QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)
+### end needed for QWebEngine
 
 import appinfo
 
@@ -63,9 +72,9 @@ jobFinished = Signal()          # (Document, Job, bool success)
 
 def activeWindow():
     """Return the currently active MainWindow.
-    
+
     Only returns None if there are no windows at all.
-    
+
     """
     if windows:
         w = QApplication.activeWindow()
@@ -75,9 +84,9 @@ def activeWindow():
 
 def openUrl(url, encoding=None):
     """Returns a Document instance for the given QUrl.
-    
+
     If there is already a document with that url, it is returned.
-    
+
     """
     d = findDocument(url)
     if not d:
@@ -94,14 +103,14 @@ def openUrl(url, encoding=None):
                 d.load(url)
         else:
             import document
-            d = document.Document.new_from_url(url, encoding)
+            d = document.EditorDocument.new_from_url(url, encoding)
     return d
 
 def findDocument(url):
     """Returns a Document instance for the given QUrl if already loaded.
-    
+
     Returns None if no document with given url exists or if the url is empty.
-    
+
     """
     if not url.isEmpty():
         for d in documents:
@@ -111,26 +120,28 @@ def findDocument(url):
 def instantiate():
     """Instantiate the global QApplication object."""
     global qApp
-    args = [os.path.abspath(sys.argv[0])] + sys.argv[1:]
-    ### on Python3, QApplication args must be byte strings
-    if sys.version_info >= (3, 0):
-        args = list(map(os.fsencode, args))
-    qApp = QApplication(args)
+    args = list(map(os.fsencode, [os.path.abspath(sys.argv[0])] + sys.argv[1:]))
+    if platform.system() == "Windows":
+        args.append("-platform") 
+        args.append("windows:fontengine=freetype") 
+    qApp = QApplication(args) 
     QApplication.setApplicationName(appinfo.name)
     QApplication.setApplicationVersion(appinfo.version)
     QApplication.setOrganizationName(appinfo.name)
     QApplication.setOrganizationDomain(appinfo.domain)
+    if sys.platform.startswith('darwin'):
+        qApp._menubar = QMenuBar()
     appInstantiated()
 
 def oninit(func):
     """Call specified function on QApplication instantiation.
-    
+
     If the QApplication already has been instantiated, the function is called
     directly.
-    
+
     As this function returns the specified function, you can use this as a
     decorator.
-    
+
     """
     if qApp:
         func()
@@ -151,16 +162,19 @@ def restart():
     python_executable = sys.executable
     if python_executable:
         args = [python_executable] + args
+    #NOTE: This deliberately uses subprocess and not job.Job
+    # It has turned out that using Job the code files are not
+    # reloaded at all.
     import subprocess
     subprocess.Popen(args)
-    
+
 def translateUI(obj, priority=0):
     """Translates texts in the object.
-    
+
     Texts are translated again if the language is changed.
     The object must have a translateUI() method.  It is
     also called by this function.
-    
+
     """
     languageChanged.connect(obj.translateUI, priority)
     obj.translateUI()
@@ -171,9 +185,9 @@ def caption(title):
 
 def filetypes(extension=None):
     """Returns a list of supported filetypes.
-    
+
     If a type matches extension, it is placed first.
-    
+
     """
     have, havenot = [], []
     for patterns, name in (
@@ -202,10 +216,10 @@ def basedir():
     import sessions
     conf = sessions.currentSessionGroup()
     if conf:
-        basedir = conf.value("basedir", "", type(""))
+        basedir = conf.value("basedir", "", str)
         if basedir:
             return basedir
-    return QSettings().value("basedir", "", type(""))
+    return QSettings().value("basedir", "", str)
 
 def settings(name):
     """Returns a QSettings object referring a file in ~/.config/frescobaldi/"""
@@ -229,3 +243,38 @@ def displayhook(obj):
     if obj is not None:
         print(repr(obj))
 
+
+_extensions = None
+
+def load_extensions(mainwindow):
+    """Called from MainWindow.__init__()"""
+    import extensions
+    global _extensions
+    _extensions = extensions.Extensions(mainwindow)
+
+def extensions():
+    """Return the global Extensions object."""
+    global _extensions
+    return _extensions
+
+
+_job_queue = None
+
+def job_queue():
+    """Return the global JobQueue object."""
+    global _job_queue
+    if _job_queue is None:
+        import job.queue
+        #TODO: save the number of runners in the Preferences
+        #NOTE: Provide code for *changing* the number of runners
+        _job_queue = job.queue.GlobalJobQueue()
+    return _job_queue
+
+_is_git_controlled = None
+
+def is_git_controlled():
+    global _is_git_controlled
+    if _is_git_controlled is None:
+        import vcs
+        _is_git_controlled = vcs.app_is_git_controlled()
+    return _is_git_controlled
