@@ -21,20 +21,19 @@
 Updates a document using convert-ly.
 """
 
-from __future__ import unicode_literals
 
 import difflib
 import textwrap
 import os
-import subprocess
-import sys
 
-from PyQt4.QtCore import QSettings, QSize
-from PyQt4.QtGui import (
-    QCheckBox, QComboBox, QDialog, QDialogButtonBox, QFileDialog, QFont, 
+from PyQt5.QtCore import QSettings, QSize
+from PyQt5.QtGui import QFont
+from PyQt5.QtWidgets import (
+    QCheckBox, QComboBox, QDialog, QDialogButtonBox, QFileDialog, QHBoxLayout,
     QGridLayout, QLabel, QLineEdit, QTabWidget, QTextBrowser, QVBoxLayout)
 
 import app
+import job
 import util
 import qutil
 import icons
@@ -68,30 +67,30 @@ def convert(mainwindow):
 class Dialog(QDialog):
     def __init__(self, parent=None):
         super(Dialog, self).__init__(parent)
-        
+
         self._info = None
         self._text = ''
         self._convertedtext = ''
         self._encoding = None
         self.mainwindow = parent
-        
+
         self.fromVersionLabel = QLabel()
         self.fromVersion = QLineEdit()
         self.reason = QLabel()
         self.toVersionLabel = QLabel()
         self.toVersion = QLineEdit()
-        self.lilyChooser = lilychooser.LilyChooser()
+        self.lilyChooser = lilychooser.LilyChooser(toolcommand='convert-ly')
         self.messages = QTextBrowser()
         self.diff = QTextBrowser(lineWrapMode=QTextBrowser.NoWrap)
         self.uni_diff = QTextBrowser(lineWrapMode=QTextBrowser.NoWrap)
         self.copyCheck = QCheckBox(checked=
             QSettings().value('convert_ly/copy_messages', True, bool))
         self.tabw = QTabWidget()
-        
+
         self.tabw.addTab(self.messages, '')
         self.tabw.addTab(self.diff, '')
         self.tabw.addTab(self.uni_diff, '')
-        
+
         self.buttons = QDialogButtonBox(
             QDialogButtonBox.Reset | QDialogButtonBox.Save |
             QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
@@ -99,10 +98,10 @@ class Dialog(QDialog):
         self.buttons.rejected.connect(self.reject)
         self.buttons.button(QDialogButtonBox.Reset).clicked.connect(self.run)
         self.buttons.button(QDialogButtonBox.Save).clicked.connect(self.saveFile)
-        
+
         layout = QVBoxLayout()
         self.setLayout(layout)
-        
+
         grid = QGridLayout()
         grid.addWidget(self.fromVersionLabel, 0, 0)
         grid.addWidget(self.fromVersion, 0, 1)
@@ -110,13 +109,13 @@ class Dialog(QDialog):
         grid.addWidget(self.toVersionLabel, 1, 0)
         grid.addWidget(self.toVersion, 1, 1)
         grid.addWidget(self.lilyChooser, 1, 3, 1, 2)
-        
+
         layout.addLayout(grid)
         layout.addWidget(self.tabw)
         layout.addWidget(self.copyCheck)
         layout.addWidget(widgets.Separator())
         layout.addWidget(self.buttons)
-        
+
         app.translateUI(self)
         qutil.saveDialogSize(self, 'convert_ly/dialog/size', QSize(600, 300))
         app.settingsChanged.connect(self.readSettings)
@@ -124,7 +123,7 @@ class Dialog(QDialog):
         self.finished.connect(self.saveCopyCheckSetting)
         self.lilyChooser.currentIndexChanged.connect(self.slotLilyPondVersionChanged)
         self.slotLilyPondVersionChanged()
-        
+
     def translateUI(self):
         self.fromVersionLabel.setText(_("From version:"))
         self.toVersionLabel.setText(_("To version:"))
@@ -138,33 +137,35 @@ class Dialog(QDialog):
         self.buttons.button(QDialogButtonBox.Reset).setText(_("Run Again"))
         self.buttons.button(QDialogButtonBox.Save).setText(_("Save as file"))
         self.setCaption()
-    
+
     def saveCopyCheckSetting(self):
         QSettings().setValue('convert_ly/copy_messages', self.copyCheck.isChecked())
-    
+
     def readSettings(self):
         font = textformats.formatData('editor').font
         self.diff.setFont(font)
         diffFont = QFont("Monospace")
         diffFont.setStyleHint(QFont.TypeWriter)
         self.uni_diff.setFont(diffFont)
-    
+
     def slotLilyPondVersionChanged(self):
         self.setLilyPondInfo(self.lilyChooser.lilyPondInfo())
-    
+
     def setCaption(self):
         version = self._info and self._info.versionString() or _("<unknown>")
         title = _("Convert-ly from LilyPond {version}").format(version=version)
         self.setWindowTitle(app.caption(title))
 
     def setLilyPondInfo(self, info):
+        if not info:
+            return
         self._info = info
         self.setCaption()
         self.toVersion.setText(info.versionString())
         self.setConvertedText()
         self.setDiffText()
         self.messages.clear()
-    
+
     def setConvertedText(self, text=''):
         self._convertedtext = text
         self.buttons.button(QDialogButtonBox.Ok).setEnabled(bool(text))
@@ -175,23 +176,23 @@ class Dialog(QDialog):
                 wrapcolumn=100))
         else:
             self.diff.clear()
-            
+
     def setDiffText(self, text=''):
         if text:
             from_filename = "current"   # TODO: maybe use real filename here
             to_filename = "converted"   # but difflib can choke on non-ascii characters,
                                         # see https://github.com/wbsoft/frescobaldi/issues/674
             difflist = list(difflib.unified_diff(
-                    self._text.split('\n'), text.split('\n'), 
+                    self._text.split('\n'), text.split('\n'),
                     from_filename, to_filename))
             diffHLstr = self.diffHighl(difflist)
             self.uni_diff.setHtml(diffHLstr)
         else:
             self.uni_diff.clear()
-    
+
     def convertedText(self):
         return self._convertedtext or ''
-    
+
     def setDocument(self, doc):
         v = documentinfo.docinfo(doc).version_string()
         if v:
@@ -203,7 +204,7 @@ class Dialog(QDialog):
         self._encoding = doc.encoding() or 'UTF-8'
         self.setConvertedText()
         self.setDiffText()
-        
+
     def run(self):
         """Runs convert-ly (again)."""
         fromVersion = self.fromVersion.text()
@@ -213,52 +214,37 @@ class Dialog(QDialog):
                 "Both 'from' and 'to' versions need to be set."))
             return
         info = self._info
-        command = info.toolcommand(info.ly_tool('convert-ly'))
+        command = info.toolcommand('convert-ly')
         command += ['-f', fromVersion, '-t', toVersion, '-']
-        
-        # if the user wants english messages, do it also here: LANGUAGE=C
-        env = None
-        if os.name == "nt":
-            # Python 2.7 subprocess on Windows chokes on unicode in env
-            env = util.bytes_environ()
-        else:
-            env = dict(os.environ)
-        if sys.platform.startswith('darwin'):
-            try:
-                del env['PYTHONHOME']
-            except KeyError:
-                pass
-            try:
-                del env['PYTHONPATH']
-            except KeyError:
-                pass
+
+        self.job = j = job.Job(command, encoding='utf-8')
         if QSettings().value("lilypond_settings/no_translation", False, bool):
-            if os.name == "nt":
-                # Python 2.7 subprocess on Windows chokes on unicode in env
-                env[b'LANGUAGE'] = b'C'
-            else:
-                env['LANGUAGE'] = 'C'
-        
-        with qutil.busyCursor():
-            try:
-                proc = subprocess.Popen(command,
-                    env = env,
-                    stdin = subprocess.PIPE,
-                    stdout = subprocess.PIPE,
-                    stderr = subprocess.PIPE)
-                out, err = proc.communicate(util.platform_newlines(self._text).encode(self._encoding))
-            except OSError as e:
-                self.messages.setPlainText(_(
-                    "Could not start {convert_ly}:\n\n"
-                    "{message}\n").format(convert_ly = command[0], message = e))
-                return
-            out = util.universal_newlines(out.decode('UTF-8'))
-            err = util.universal_newlines(err.decode('UTF-8'))
-            self.messages.setPlainText(err)
-            self.setConvertedText(out)
-            self.setDiffText(out)
-            if not out or self._convertedtext == self._text:
-                self.messages.append('\n' + _("The document has not been changed."))
+            j.environment['LANG'] = 'C'
+            j.environment['LC_ALL'] = 'C'
+        else:
+            j.environment.pop('LANG', None)
+            j.environment.pop('LC_ALL', None)
+
+        j.done.connect(self.slotJobDone)
+        app.job_queue().add_job(j, 'generic')
+        j._process.write(self._text.encode('utf-8'))
+        j._process.closeWriteChannel()
+
+    def slotJobDone(self):
+        j = self.job
+        if not j.success and j.failed_to_start():
+            self.messages.setPlainText(_(
+                "Could not start {convert_ly}:\n\n"
+                "{message}\n").format(convert_ly = j.command[0],
+                    message = j.error))
+            return
+        out = j.stdout()
+        err = j.stderr()
+        self.messages.setPlainText(err)
+        self.setConvertedText(out)
+        self.setDiffText(out)
+        if not out or self._convertedtext == self._text:
+            self.messages.append('\n' + _("The document has not been changed."))
 
     def saveFile(self):
         """Save content in tab as file"""
@@ -268,7 +254,7 @@ class Dialog(QDialog):
         filename = os.path.splitext(orgname)[0] + '['+tabdata.filename+']'+'.'+tabdata.ext
         caption = app.caption(_("dialog title", "Save File"))
         filetypes = '{0} (*.txt);;{1} (*.htm);;{2} (*)'.format(_("Text Files"), _("HTML Files"), _("All Files"))
-        filename = QFileDialog.getSaveFileName(self.mainwindow, caption, filename, filetypes)
+        filename = QFileDialog.getSaveFileName(self.mainwindow, caption, filename, filetypes)[0]
         if not filename:
             return False # cancelled
         with open(filename, 'wb') as f:
@@ -282,7 +268,7 @@ class Dialog(QDialog):
             return FileInfo('html-diff', 'html', self.diff.toHtml())
         elif index == 2:
             return FileInfo('uni-diff', 'diff', self.uni_diff.toPlainText())
-            
+
     def diffHighl(self, difflist):
         """Return highlighted version of input."""
         result = []
@@ -304,5 +290,3 @@ class FileInfo():
         self.filename = filename
         self.ext = ext
         self.text = text
-		
-		 
