@@ -26,6 +26,7 @@ import functools
 import sys
 import os
 
+import app
 import appinfo
 
 
@@ -47,7 +48,10 @@ def app_version():
 
 @_catch_unknown
 def sip_version():
-    import sip
+    try:
+        import sip
+    except ImportError:
+        import PyQt5.sip as sip
     return sip.SIP_VERSION_STR
 
 @_catch_unknown
@@ -89,11 +93,10 @@ if sys.platform.startswith('darwin'):
     @_catch_unknown
     def mac_installation_kind():
         import macosx
-        if macosx.inside_app_bundle():
-            if os.path.islink(os.getcwd() + '/../MacOS/python'):
-                return 'lightweight app bundle'
-            else:
-                return 'standalone app bundle'
+        if macosx.inside_lightweight_app_bundle():
+            return 'lightweight .app bundle'
+        elif macosx.inside_app_bundle():
+            return 'standalone .app bundle'
         else:
             return 'command line'
 
@@ -101,7 +104,16 @@ if sys.platform.startswith('darwin'):
 def version_info_named():
     """Yield all the relevant names and their version string."""
     yield appinfo.appname, appinfo.version
+    yield "Extension API", appinfo.extension_api
     yield "Python", python_version()
+    if app.is_git_controlled():
+        import vcs
+        repo = vcs.app_repo
+        yield "Git branch", repo.active_branch()
+        commit = repo.run_command(
+            'log',
+            ['-n', '1', '--format=format:%h'])
+        yield "on commit", commit[0]
     yield "python-ly", ly_version()
     yield "Qt", qt_version()
     yield "PyQt", pyqt_version()
@@ -116,5 +128,3 @@ def version_info_named():
 def version_info_string(separator='\n'):
     """Return all version names as a string, joint with separator."""
     return separator.join(map("{0[0]}: {0[1]}".format, version_info_named()))
-
-
