@@ -27,8 +27,7 @@ import os
 from PyQt5.QtCore import QSettings, Qt, QUrl
 from PyQt5.QtGui import QKeySequence
 from PyQt5.QtPrintSupport import QPrintDialog, QPrinter
-from PyQt5.QtWebKit import QWebSettings
-from PyQt5.QtWebKitWidgets import QWebPage, QWebView
+from PyQt5.QtWebEngineWidgets import QWebEnginePage, QWebEngineView, QWebEngineSettings
 from PyQt5.QtWidgets import QComboBox, QMenu, QToolBar, QVBoxLayout, QWidget
 
 import app
@@ -45,33 +44,29 @@ class Browser(QWidget):
     """LilyPond documentation browser widget."""
     def __init__(self, dockwidget):
         super(Browser, self).__init__(dockwidget)
-        
+
         layout = QVBoxLayout(spacing=0)
         layout.setContentsMargins(0, 0, 0, 0)
         self.setLayout(layout)
-        
+
         self.toolbar = tb = QToolBar()
-        self.webview = QWebView(contextMenuPolicy=Qt.CustomContextMenu)
+        self.webview = QWebEngineView(self, contextMenuPolicy=Qt.CustomContextMenu)
+        self.webview.setPage(WebEnginePage(self.webview))
         self.chooser = QComboBox(sizeAdjustPolicy=QComboBox.AdjustToContents)
         self.search = SearchEntry(maximumWidth=200)
-        
+
         layout.addWidget(self.toolbar)
         layout.addWidget(self.webview)
-        
+
         ac = dockwidget.actionCollection
         ac.help_back.triggered.connect(self.webview.back)
         ac.help_forward.triggered.connect(self.webview.forward)
         ac.help_home.triggered.connect(self.showHomePage)
         ac.help_print.triggered.connect(self.slotPrint)
-        
-        self.webview.page().setNetworkAccessManager(lilydoc.network.accessmanager())
-        self.webview.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
-        self.webview.page().linkClicked.connect(self.openUrl)
-        self.webview.page().setForwardUnsupportedContent(True)
-        self.webview.page().unsupportedContent.connect(self.slotUnsupported)
+
         self.webview.urlChanged.connect(self.slotUrlChanged)
         self.webview.customContextMenuRequested.connect(self.slotShowContextMenu)
-        
+
         tb.addAction(ac.help_back)
         tb.addAction(ac.help_forward)
         tb.addSeparator()
@@ -80,53 +75,54 @@ class Browser(QWidget):
         tb.addSeparator()
         tb.addWidget(self.chooser)
         tb.addWidget(self.search)
-        
+
         self.chooser.activated[int].connect(self.showHomePage)
-        self.search.textEdited.connect(self.slotSearchChanged)
+        self.search.textChanged.connect(self.slotSearchChanged)
         self.search.returnPressed.connect(self.slotSearchReturnPressed)
         dockwidget.mainwindow().iconSizeChanged.connect(self.updateToolBarSettings)
         dockwidget.mainwindow().toolButtonStyleChanged.connect(self.updateToolBarSettings)
-        
+
         app.settingsChanged.connect(self.readSettings)
         self.readSettings()
         self.loadDocumentation()
         self.showInitialPage()
         app.settingsChanged.connect(self.loadDocumentation)
         app.translateUI(self)
-    
+
     def readSettings(self):
         s = QSettings()
         s.beginGroup("documentation")
         ws = self.webview.page().settings()
         family = s.value("fontfamily", self.font().family(), str)
         size = s.value("fontsize", 16, int)
-        ws.setFontFamily(QWebSettings.StandardFont, family)
-        ws.setFontSize(QWebSettings.DefaultFontSize, size)
+        ws.setFontFamily(QWebEngineSettings.StandardFont, family)
+        ws.setFontSize(QWebEngineSettings.DefaultFontSize, size)
         fixed = textformats.formatData('editor').font
-        ws.setFontFamily(QWebSettings.FixedFont, fixed.family())
-        ws.setFontSize(QWebSettings.DefaultFixedFontSize, fixed.pointSizeF() * 96 / 72)
-        
+        ws.setFontFamily(QWebEngineSettings.FixedFont, fixed.family())
+        ws.setFontSize(QWebEngineSettings.DefaultFixedFontSize, fixed.pointSizeF() * 96 / 72)
+        self.webview.page().profile().setHttpAcceptLanguage(','.join(lilydoc.network.langs()))
+
     def keyPressEvent(self, ev):
         if ev.text() == "/":
             self.search.setFocus()
         else:
             super(Browser, self).keyPressEvent(ev)
-        
+
     def translateUI(self):
         try:
             self.search.setPlaceholderText(_("Search..."))
         except AttributeError:
             pass # not in Qt 4.6
-    
+
     def showInitialPage(self):
         """Shows the preferred start page.
-        
+
         If a local documentation instance already has a suitable version,
         just loads it. Otherwise connects to the allLoaded signal, that is
         emitted when all the documentation instances have loaded their version
         information and then shows the start page (if another page wasn't yet
         loaded).
-        
+
         """
         if self.webview.url().isEmpty():
             docs = lilydoc.manager.docs()
@@ -146,7 +142,7 @@ class Browser(QWidget):
                 index = len(docs) - 1
             self.chooser.setCurrentIndex(index)
             self.showHomePage()
-    
+
     def loadDocumentation(self):
         """Puts the available documentation instances in the combobox."""
         i = self.chooser.currentIndex()
@@ -162,42 +158,42 @@ class Browser(QWidget):
         if not lilydoc.manager.loaded():
             lilydoc.manager.allLoaded.connect(self.loadDocumentation, -1)
             return
-        
+
     def updateToolBarSettings(self):
         mainwin = self.parentWidget().mainwindow()
         self.toolbar.setIconSize(mainwin.iconSize())
         self.toolbar.setToolButtonStyle(mainwin.toolButtonStyle())
-        
+
     def showManual(self):
         """Invoked when the user presses F1."""
         self.slotHomeFrescobaldi() # TEMP
-        
+
     def slotUrlChanged(self):
         ac = self.parentWidget().actionCollection
         ac.help_back.setEnabled(self.webview.history().canGoBack())
         ac.help_forward.setEnabled(self.webview.history().canGoForward())
-    
+
     def openUrl(self, url):
         if url.path().endswith(('.ily', '.lyi', '.ly')):
             self.sourceViewer().showReply(lilydoc.network.get(url))
         else:
             self.webview.load(url)
-    
+
     def slotUnsupported(self, reply):
         helpers.openUrl(reply.url())
-    
+
     def slotSearchChanged(self):
         text = self.search.text()
         if not text.startswith(':'):
-            self.webview.page().findText(text, QWebPage.FindWrapsAroundDocument)
-    
+            self.webview.page().findText(text)
+
     def slotSearchReturnPressed(self):
         text = self.search.text()
         if not text.startswith(':'):
             self.slotSearchChanged()
         else:
             pass # TODO: implement full doc search
-    
+
     def sourceViewer(self):
         try:
             return self._sourceviewer
@@ -205,14 +201,14 @@ class Browser(QWidget):
             from . import sourceviewer
             self._sourceviewer = sourceviewer.SourceViewer(self)
             return self._sourceviewer
-    
+
     def showHomePage(self):
         """Shows the homepage of the LilyPond documentation."""
         i = self.chooser.currentIndex()
         if i < 0:
             i = 0
         doc = lilydoc.manager.docs()[i]
-        
+
         url = doc.home()
         if doc.isLocal():
             path = url.toLocalFile()
@@ -224,28 +220,31 @@ class Browser(QWidget):
                         break
             url = QUrl.fromLocalFile(path + '.html')
         self.webview.load(url)
-    
+
     def slotPrint(self):
-        printer = QPrinter()
+        printer = self._printer = QPrinter()
         dlg = QPrintDialog(printer, self)
         dlg.setWindowTitle(app.caption(_("Print")))
         if dlg.exec_():
-            self.webview.print_(printer)
-    
+            self.webview.page().print(printer, self.slotPrintingDone)
+
+    def slotPrintingDone(self, success):
+        del self._printer
+
     def slotShowContextMenu(self, pos):
-        hit = self.webview.page().currentFrame().hitTestContent(pos)
+        d = self.webview.page().contextMenuData()
         menu = QMenu()
-        if hit.linkUrl().isValid():
-            a = self.webview.pageAction(QWebPage.CopyLinkToClipboard)
+        if d.linkUrl().isValid():
+            a = self.webview.pageAction(QWebEnginePage.CopyLinkToClipboard)
             a.setIcon(icons.get("edit-copy"))
             a.setText(_("Copy &Link"))
             menu.addAction(a)
             menu.addSeparator()
             a = menu.addAction(icons.get("window-new"), _("Open Link in &New Window"))
-            a.triggered.connect((lambda url: lambda: self.slotNewWindow(url))(hit.linkUrl()))
+            a.triggered.connect((lambda url: lambda: self.slotNewWindow(url))(d.linkUrl()))
         else:
-            if hit.isContentSelected():
-                a = self.webview.pageAction(QWebPage.Copy)
+            if d.selectedText():
+                a = self.webview.pageAction(QWebEnginePage.Copy)
                 a.setIcon(icons.get("edit-copy"))
                 a.setText(_("&Copy"))
                 menu.addAction(a)
@@ -254,7 +253,7 @@ class Browser(QWidget):
             a.triggered.connect((lambda url: lambda: self.slotNewWindow(url))(self.webview.url()))
         if menu.actions():
             menu.exec_(self.webview.mapToGlobal(pos))
-    
+
     def slotNewWindow(self, url):
         helpers.openUrl(url)
 
@@ -279,3 +278,12 @@ class SearchEntry(widgets.lineedit.LineEdit):
             super(SearchEntry, self).keyPressEvent(ev)
 
 
+class WebEnginePage(QWebEnginePage):
+    """QWebEnginePage that shows LY files using the source viewer."""
+    def acceptNavigationRequest(self, url, type, isMainFrame):
+        if url.path().endswith(('.ily', '.lyi', '.ly')):
+            self.view().parent().sourceViewer().showReply(lilydoc.network.get(url))
+            return False
+        return super().acceptNavigationRequest(url, type, isMainFrame)
+
+
