@@ -31,8 +31,9 @@ from PyQt5.QtWidgets import QAction, QApplication, QMessageBox
 import app
 import actioncollection
 import actioncollectionmanager
-import jobmanager
-import jobattributes
+import documentinfo
+import job.attributes
+import job.lilypond
 import plugin
 import icons
 import signals
@@ -42,12 +43,12 @@ import variables
 
 def engraver(mainwindow):
     return Engraver.instance(mainwindow)
-    
+
 
 class Engraver(plugin.MainWindowPlugin):
-    
+
     stickyChanged = signals.Signal()    # Document
-    
+
     def __init__(self, mainwindow):
         self._currentStickyDocument = None
         ac = self.actionCollection = Actions()
@@ -61,7 +62,6 @@ class Engraver(plugin.MainWindowPlugin):
         ac.engrave_abort.triggered.connect(self.engraveAbort)
         ac.engrave_autocompile.toggled.connect(self.engraveAutoCompileToggled)
         ac.engrave_open_lilypond_datadir.triggered.connect(self.openLilyPondDatadir)
-        ac.engrave_show_available_fonts.triggered.connect(self.showAvailableFonts)
         mainwindow.currentDocumentChanged.connect(self.updateActions)
         app.jobStarted.connect(self.updateActions)
         app.jobFinished.connect(self.updateActions)
@@ -74,7 +74,7 @@ class Engraver(plugin.MainWindowPlugin):
         self.loadSettings()
         app.languageChanged.connect(self.updateStickyActionText)
         self.updateStickyActionText()
-        
+
     def document(self):
         """Return the Document that should be engraved."""
         doc = self.stickyDocument()
@@ -89,18 +89,18 @@ class Engraver(plugin.MainWindowPlugin):
                     except IOError:
                         pass
         return doc
-                
+
     def runningJob(self):
         """Returns a Job for the sticky or current document if that is running."""
         doc = self.document()
-        job = jobmanager.job(doc)
-        if job and job.is_running() and not jobattributes.get(job).hidden:
-            return job
-    
+        j = job.manager.job(doc)
+        if j and j.is_running() and not job.attributes.get(j).hidden:
+            return j
+
     def updateActions(self):
-        job = jobmanager.job(self.document())
-        running = bool(job and job.is_running())
-        visible = running and not jobattributes.get(job).hidden
+        j = job.manager.job(self.document())
+        running = bool(j and j.is_running())
+        visible = running and not job.attributes.get(j).hidden
         ac = self.actionCollection
         ac.engrave_preview.setEnabled(not visible)
         ac.engrave_publish.setEnabled(not visible)
@@ -109,49 +109,49 @@ class Engraver(plugin.MainWindowPlugin):
         ac.engrave_runner.setIcon(icons.get('process-stop' if visible else 'lilypond-run'))
         ac.engrave_runner.setToolTip(_("Abort engraving job") if visible else
                     _("Engrave (preview; press Shift for custom)"))
-    
-    def openDefaultView(self, document, job, success):
+
+    def openDefaultView(self, document, j, success):
         """Called when a job finishes.
-        
+
         Open the default viewer for the created files if the user has the
         preference for this set.
-        
+
         """
-        if (success and jobattributes.get(job).mainwindow is self.mainwindow()
+        if (success and job.attributes.get(j).mainwindow is self.mainwindow()
                 and QSettings().value("lilypond_settings/open_default_view", True, bool)):
-            
+
             # which files were created by this job?
             import resultfiles
             extensions = set(os.path.splitext(filename)[1].lower()
                 for filename in resultfiles.results(document).files_lastjob())
-            
+
             mgr = panelmanager.manager(self.mainwindow())
             if '.svg' in extensions or '.svgz' in extensions:
                 mgr.svgview.activate()
             elif '.pdf' in extensions:
                 mgr.musicview.activate()
-    
+
     def engraveRunner(self):
-        job = self.runningJob()
-        if job:
-            job.abort()
+        j = self.runningJob()
+        if j:
+            j.abort()
         elif QApplication.keyboardModifiers() & Qt.SHIFT:
             self.engraveCustom()
         else:
             self.engravePreview()
-    
+
     def engravePreview(self):
         """Starts an engrave job in preview mode (with point and click turned on)."""
         self.engrave('preview')
-    
+
     def engravePublish(self):
         """Starts an engrave job in publish mode (with point and click turned off)."""
         self.engrave('publish')
-        
+
     def engraveLayoutControl(self):
         """Starts an engrave job in debug mode (using the settings in the debug tool)."""
         self.engrave('layout-control')
-        
+
     def engraveCustom(self):
         """Opens a dialog to configure the job before starting it."""
         try:
@@ -166,45 +166,49 @@ class Engraver(plugin.MainWindowPlugin):
         if dlg.exec_():
             self.saveDocumentIfDesired()
             self.runJob(dlg.getJob(doc), doc)
-    
+
     def engrave(self, mode='preview', document=None, may_save=True):
         """Starts an engraving job.
-        
-        The mode can be 'preview', 'publish', or 'layout-control'. The last 
-        one uses the settings in the Layout Control Options panel. The 
+
+        The mode can be 'preview', 'publish', or 'layout-control'. The last
+        one uses the settings in the Layout Control Options panel. The
         default mode is 'preview'.
-        
+
         If document is not specified, it is either the sticky or current
         document.
-        
+
         If may_save is False, the document will not be saved before running
         LilyPond, even if the preference setting "save document before LilyPond
         is run" is enabled.
-        
+
         """
-        if mode == 'preview':
-            args = ['-dpoint-and-click']
-        elif mode == 'publish':
-            args = None     # command.defaultJob() will handle publish mode
-        elif mode == 'layout-control':
-            args = panelmanager.manager(
-                    self.mainwindow()).layoutcontrol.widget().preview_options()
+        job_class = (
+            job.lilypond.PreviewJob if mode == 'preview'
+            else job.lilypond.PublishJob if mode == 'publish'
+            else job.lilypond.LayoutControlJob
+        )
+        # TODO: Try to move this argument creation into
+        # LayoutControlJob's constructor. However, somehow this has to
+        # obtain access to the mainwindow.
+        args = (
+            panelmanager.manager(
+                self.mainwindow()).layoutcontrol.widget().preview_options()
+            if mode == 'layout-control' else None)
         doc = document or self.document()
         if may_save:
             self.saveDocumentIfDesired()
-        from . import command
-        self.runJob(command.defaultJob(doc, args), doc)
-    
+        self.runJob(job_class(doc, args), doc)
+
     def engraveAbort(self):
-        job = jobmanager.job(self.document())
-        if job and job.is_running():
-            job.abort()
-    
+        j = job.manager.job(self.document())
+        if j and j.is_running():
+            j.abort()
+
     def saveDocumentIfDesired(self):
         """Saves the current document if desired and it makes sense.
-        
+
         (i.e. the document is modified and has a local filename)
-        
+
         """
         if QSettings().value("lilypond_settings/save_on_run", False, bool):
             doc = self.mainwindow().currentDocument()
@@ -216,14 +220,14 @@ class Engraver(plugin.MainWindowPlugin):
 
     def queryCloseDocument(self, doc):
         """Return True whether a document can be closed.
-        
+
         When no job is running, True is immediately returned.
         When a job is running, the user is asked whether to abort the job (not
         for autocompile ("hidden") jobs).
-        
+
         """
-        job = jobmanager.job(doc)
-        if not job or not job.is_running() or jobattributes.get(job).hidden:
+        j = job.manager.job(doc)
+        if not j or not j.is_running() or job.attributes.get(j).hidden:
             return True
         msgbox = QMessageBox(QMessageBox.Warning,
             _("Warning"),
@@ -233,24 +237,24 @@ class Engraver(plugin.MainWindowPlugin):
             self.mainwindow())
         abort_button = msgbox.button(QMessageBox.Abort)
         signal = lambda: abort_button.click()
-        job.done.connect(signal)
+        j.done.connect(signal)
         msgbox.exec_()
-        job.done.disconnect(signal)
+        j.done.disconnect(signal)
         return msgbox.clickedButton() == abort_button
 
-    def runJob(self, job, document):
+    def runJob(self, j, document):
         """Runs the engraving job on behalf of document."""
-        jobattributes.get(job).mainwindow = self.mainwindow()
+        job.attributes.get(j).mainwindow = self.mainwindow()
         # cancel running job, that would be an autocompile job
-        rjob = jobmanager.job(document)
+        rjob = job.manager.job(document)
         if rjob and rjob.is_running():
             rjob.abort()
-        jobmanager.manager(document).start_job(job)
-    
+        job.manager.manager(document).start_job(j)
+
     def stickyToggled(self):
         """Called when the user toggles the 'Sticky' action."""
         self.setStickyDocument(None if self.stickyDocument() else self.mainwindow().currentDocument())
-    
+
     def setStickyDocument(self, doc=None):
         """Sticks to the given document or removes the 'stick' when None."""
         cur = self._currentStickyDocument
@@ -266,11 +270,11 @@ class Engraver(plugin.MainWindowPlugin):
         self.actionCollection.engrave_sticky.setChecked(bool(doc))
         self.updateStickyActionText()
         self.updateActions()
-        
+
     def stickyDocument(self):
         """Returns the document currently marked as 'Sticky', if any."""
         return self._currentStickyDocument
-    
+
     def slotUnStickDocument(self):
         """Called when the document that is currently sticky closes or reloads."""
         self.setStickyDocument(None)
@@ -283,34 +287,26 @@ class Engraver(plugin.MainWindowPlugin):
         else:
             text = _("&Always Engrave This Document")
         self.actionCollection.engrave_sticky.setText(text)
-    
+
     def engraveAutoCompileToggled(self, enabled):
         """Called when the user toggles autocompile on/off."""
         from . import autocompile
         autocompile.AutoCompiler.instance(self.mainwindow()).setEnabled(enabled)
-    
+
     def openLilyPondDatadir(self):
         """Menu action Open LilyPond Data Directory."""
-        from . import command
-        info = command.info(self.mainwindow().currentDocument())
+        info = documentinfo.lilyinfo(self.mainwindow().currentDocument())
         datadir = info.datadir()
         if datadir:
             import helpers
             helpers.openUrl(QUrl.fromLocalFile(datadir))
-    
-    def showAvailableFonts(self):
-        """Menu action Show Available Fonts."""
-        from . import command
-        info = command.info(self.mainwindow().currentDocument())
-        from . import lytools
-        lytools.show_available_fonts(self.mainwindow(), info)
-        
+
     def slotDocumentClosed(self, doc):
         """Called when the user closes a document. Aborts a running Job."""
-        job = jobmanager.job(doc)
-        if job and job.is_running():
-            job.abort()
-        
+        j = job.manager.job(doc)
+        if j and j.is_running():
+            j.abort()
+
     def slotSessionChanged(self):
         """Called when the session is changed."""
         import sessions
@@ -321,7 +317,7 @@ class Engraver(plugin.MainWindowPlugin):
                 d = app.findDocument(url)
                 if d:
                     self.setStickyDocument(d)
-    
+
     def slotSaveSessionData(self):
         """Called when a session needs to save data."""
         import sessions
@@ -332,29 +328,29 @@ class Engraver(plugin.MainWindowPlugin):
                 g.setValue("sticky_url", d.url())
             else:
                 g.remove("sticky_url")
-    
+
     def saveSettings(self):
         """Save the state of some actions."""
         ac = self.actionCollection
         s = QSettings()
         s.beginGroup("engraving")
         s.setValue("autocompile", ac.engrave_autocompile.isChecked())
-        
+
     def loadSettings(self):
         """Load the state of some actions."""
         ac = self.actionCollection
         s = QSettings()
         s.beginGroup("engraving")
         ac.engrave_autocompile.setChecked(s.value("autocompile", False, bool))
-    
-    def checkLilyPondInstalled(self, document, job, success):
+
+    def checkLilyPondInstalled(self, document, j, success):
         """Called when LilyPond is run for the first time.
-        
+
         Displays a helpful dialog if the process failed to start.
-        
+
         """
         app.jobFinished.disconnect(self.checkLilyPondInstalled)
-        if not success and job.failed_to_start():
+        if not success and j.failed_to_start():
             QMessageBox.warning(self.mainwindow(),
                 _("No LilyPond installation found"), _(
                 "Frescobaldi uses LilyPond to engrave music, "
@@ -367,7 +363,7 @@ class Engraver(plugin.MainWindowPlugin):
 
 class Actions(actioncollection.ActionCollection):
     name = "engrave"
-    
+
     def createActions(self, parent=None):
         self.engrave_sticky = QAction(parent)
         self.engrave_sticky.setCheckable(True)
@@ -379,21 +375,20 @@ class Actions(actioncollection.ActionCollection):
         self.engrave_abort = QAction(parent)
         self.engrave_autocompile = QAction(parent)
         self.engrave_autocompile.setCheckable(True)
-        self.engrave_show_available_fonts = QAction(parent)
         self.engrave_open_lilypond_datadir = QAction(parent)
-        
+
         self.engrave_preview.setShortcut(QKeySequence(Qt.CTRL + Qt.Key_M))
         self.engrave_publish.setShortcut(QKeySequence(Qt.CTRL + Qt.SHIFT + Qt.Key_P))
         self.engrave_custom.setShortcut(QKeySequence(Qt.CTRL + Qt.SHIFT + Qt.Key_M))
         self.engrave_abort.setShortcut(QKeySequence(Qt.CTRL + Qt.Key_Pause))
-        
+
         self.engrave_sticky.setIcon(icons.get('pushpin'))
         self.engrave_preview.setIcon(icons.get('lilypond-run'))
         self.engrave_publish.setIcon(icons.get('lilypond-run'))
         self.engrave_debug.setIcon(icons.get('lilypond-run'))
         self.engrave_custom.setIcon(icons.get('lilypond-run'))
         self.engrave_abort.setIcon(icons.get('process-stop'))
-        
+
 
     def translateUI(self):
         self.engrave_runner.setText(_("Engrave"))
@@ -405,6 +400,3 @@ class Actions(actioncollection.ActionCollection):
         self.engrave_abort.setText(_("Abort Engraving &Job"))
         self.engrave_autocompile.setText(_("Automatic E&ngrave"))
         self.engrave_open_lilypond_datadir.setText(_("Open LilyPond &Data Directory"))
-        self.engrave_show_available_fonts.setText(_("Show Available &Fonts..."))
-        
-        
