@@ -30,31 +30,34 @@ import icons
 
 
 class LilyChooser(QComboBox):
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, toolcommand=None):
         super(LilyChooser, self).__init__(parent)
         self._infos = []
+        self._toolcommand = toolcommand
         app.translateUI(self)
         app.settingsChanged.connect(self.load)
         self.load()
-    
+
     def translateUI(self):
         self.setToolTip(_("Choose the desired LilyPond version."""))
-    
+
     def setLilyPondInfo(self, info):
         """Set the current LilyPond info (one of lilypondinfo.infos())."""
         try:
             self.setCurrentIndex(self._infos.index(info))
         except IndexError:
             pass
-    
+
     def lilyPondInfo(self):
         """Get the current LilyPond info."""
         if self._infos:
             return self._infos[self.currentIndex()]
-    
+
     def load(self):
         """Load the available LilyPond infos."""
         infos = lilypondinfo.infos() or [lilypondinfo.default()]
+        if self._toolcommand:
+            infos = [info for info in infos if info.toolcommand(self._toolcommand)]
         infos.sort(key = lambda i: i.version() or (999,))
         cur = self._infos[self.currentIndex()] if self._infos else lilypondinfo.preferred()
         self._infos = infos
