@@ -45,51 +45,51 @@ class LogTool(panel.Panel):
         mainwindow.addDockWidget(Qt.BottomDockWidgetArea, self)
         app.jobStarted.connect(self.slotJobStarted)
         app.jobFinished.connect(self.slotJobFinished)
-    
+
     def translateUI(self):
         self.setWindowTitle(_("LilyPond Log"))
         self.toggleViewAction().setText(_("LilyPond &Log"))
-        
+
     def createWidget(self):
         from . import logwidget
         return logwidget.LogWidget(self)
-    
-    def slotJobStarted(self, doc, job):
+
+    def slotJobStarted(self, doc, j):
         """Called whenever job starts, decides whether to follow it and show the log."""
-        import jobattributes
-        jattrs = jobattributes.get(job)
+        import job
+        jattrs = job.attributes.get(j)
         if doc == self.mainwindow().currentDocument() or self.mainwindow() == jattrs.mainwindow:
             self.widget().switchDocument(doc)
             if not jattrs.hidden and QSettings().value("log/show_on_start", True, bool):
                 self.show()
 
-    def slotJobFinished(self, document, job, success):
-        import jobattributes
-        if (not success and not job.is_aborted()
-                and not jobattributes.get(job).hidden
+    def slotJobFinished(self, document, j, success):
+        import job
+        if (not success and not j.is_aborted()
+                and not job.attributes.get(j).hidden
                 and document == self.mainwindow().currentDocument()):
             self.show()
-    
+
     def slotNextError(self):
         """Jumps to the position pointed to by the next error message."""
         self.activate()
         self.widget().gotoError(1)
-    
+
     def slotPreviousError(self):
         """Jumps to the position pointed to by the next error message."""
         self.activate()
         self.widget().gotoError(-1)
-        
+
 
 class Actions(actioncollection.ActionCollection):
     name = "logtool"
     def createActions(self, parent=None):
         self.log_next_error = QAction(parent)
         self.log_previous_error = QAction(parent)
-        
+
         self.log_next_error.setShortcut(QKeySequence("Ctrl+E"))
         self.log_previous_error.setShortcut(QKeySequence("Ctrl+Shift+E"))
-        
+
     def translateUI(self):
         self.log_next_error.setText(_("Next Error Message"))
         self.log_previous_error.setText(_("Previous Error Message"))
@@ -100,4 +100,3 @@ class Actions(actioncollection.ActionCollection):
 def _log_errors(document):
     from . import errors
     errors.errors(document)
-
