@@ -22,10 +22,6 @@ Entry point of Frescobaldi.
 """
 
 
-import sip
-sip.setapi("QString", 2)
-sip.setapi("QVariant", 2)
-
 import os
 import re
 import sys
@@ -38,16 +34,16 @@ import appinfo             # Information about our application
 import app              # Instantiate global signals etc
 import install          # Update QSettings structure etc. if needed
 import guistyle         # Setup GUI style
-import po.setup         # Setup language
+import i18n.setup       # Setup language
 import remote           # IPC with other Frescobaldi instances
 
 
 def parse_commandline():
     """Parses the command line; returns options and filenames.
-    
+
     If --version, --help or invalid options were given, the application will
     exit.
-    
+
     """
     import argparse
     argparse._ = _ # let argparse use our translations
@@ -72,9 +68,11 @@ def parse_commandline():
         help=_("List the session names and exit"))
     parser.add_argument('-n', '--new', action="store_true", default=False,
         help=_("Always start a new instance"))
-    parser.add_argument('files', metavar=_("file"), nargs='*', 
+    parser.add_argument('--python-ly', type=str, metavar=_("STR"), default="",
+        help=_("Path to python-ly"))
+    parser.add_argument('files', metavar=_("file"), nargs='*',
         help=_("File to be opened"))
-    
+
     # Make sure debugger options are recognized as valid. These are passed automatically
     # from PyDev in Eclipse to the inferior process.
     if "pydevd" in sys.modules:
@@ -110,63 +108,27 @@ def url(arg):
         return QUrl.fromLocalFile(os.path.abspath(arg))
 
 
-def patch_pyqt():
-    """Patch PyQt classes to strip trailing null characters on Python 2
-
-    It works around a bug triggered by Unicode characters above 0xFFFF.
-    """
-    if sys.version_info >= (3, 3):
-        return
-
-    old_toLocalFile = QUrl.toLocalFile
-    QUrl.toLocalFile = lambda url: old_toLocalFile(url).rstrip('\0')
-
-    old_toString = QUrl.toString
-    QUrl.toString = lambda *args: old_toString(*args).rstrip('\0')
-
-    old_path = QUrl.path
-    QUrl.path = lambda self: old_path(self).rstrip('\0')
-
-    old_arguments = QApplication.arguments
-    QApplication.arguments = staticmethod(
-        lambda: [arg.rstrip('\0') for arg in old_arguments()])
-
-    from PyQt5.QtWidgets import QFileDialog
-    old_getOpenFileNames = QFileDialog.getOpenFileNames
-    QFileDialog.getOpenFileNames = staticmethod(
-        lambda *args: [f.rstrip('\0') for f in old_getOpenFileNames(*args)])
-
-    old_getOpenFileName = QFileDialog.getOpenFileName
-    QFileDialog.getOpenFileName = staticmethod(
-        lambda *args: old_getOpenFileName(*args).rstrip('\0'))
-
-    old_getSaveFileName = QFileDialog.getSaveFileName
-    QFileDialog.getSaveFileName = staticmethod(
-        lambda *args: old_getSaveFileName(*args).rstrip('\0'))
-
-
 def check_ly():
     """Check if ly is installed and has the correct version.
-    
+
     If python-ly is not available or too old, we display a critical message
     and exit.  In the future we will probably remove this function.
     In a good software distribution this should never happen, but it can happen
     when a user leaves the old stale 'ly' package in the frescobaldi_app
     directory, or has not installed python-ly at all.
-    
+
     Because that yields unexpected behaviour and error messages, we better
     check for the availability of the 'ly' module beforehand.
     Importing ly.pkginfo is not expensive.
-    
+
     """
-    ly_required = (0, 9, 4)
     try:
         import ly.pkginfo
         version = tuple(map(int, re.findall(r"\d+", ly.pkginfo.version)))
     except (ImportError, AttributeError):
         pass
     else:
-        if version >= ly_required:
+        if version >= appinfo.required_python_ly_version:
             return
     from PyQt5.QtWidgets import QMessageBox
     QMessageBox.critical(None, _("Can't run Frescobaldi"), _(
@@ -176,30 +138,36 @@ def check_ly():
         "If you did install python-ly correctly, please remove the old\n"
         "ly package from the frescobaldi_app directory, or completely\n"
         "remove and then reinstall Frescobaldi."
-    ).format(version=".".join(format(d) for d in ly_required)))
+    ).format(version=".".join(format(d) for d in appinfo.required_python_ly_version)))
     sys.exit(1)
 
 
 def main():
     """Main function."""
     args = parse_commandline()
-    
+
     if args.version_debug:
         import debuginfo
         sys.stdout.write(debuginfo.version_info_string() + '\n')
         sys.exit(0)
-    
+
+    if args.python_ly:
+        # The python-ly path has to be inserted at the *second* position
+        # because the first element in sys.path is the directory of the invoked
+        # script (an information we need in determining if Frescobaldi is run
+        # from its Git repository)
+        sys.path.insert(1, args.python_ly)
+
     check_ly()
-    patch_pyqt()
-    
+
     if args.list_sessions:
         import sessions
         for name in sessions.sessionNames():
             sys.stdout.write(name + '\n')
         sys.exit(0)
-    
+
     urls = list(map(url, args.files))
-    
+
     if not app.qApp.isSessionRestored():
         if not args.new and remote.enabled():
             api = remote.get()
@@ -207,7 +175,7 @@ def main():
                 api.command_line(args, urls)
                 api.close()
                 sys.exit(0)
-    
+
         if QSettings().value("splash_screen", True, bool):
             import splashscreen
             splashscreen.show()
@@ -215,9 +183,9 @@ def main():
     # application icon
     import icons
     QApplication.setWindowIcon(icons.get("frescobaldi"))
-    
+
     QTimer.singleShot(0, remote.setup)  # Start listening for IPC
-    
+
     import mainwindow       # contains MainWindow class
     import session          # Initialize QSessionManager support
     import sessions         # Initialize our own named session support
@@ -228,33 +196,35 @@ def main():
     import musicpos         # shows music time in statusbar
     import autocomplete     # auto-complete input
     import wordboundary     # better wordboundary behaviour for the editor
-    
+
     if sys.platform.startswith('darwin'):
         import macosx.setup
         macosx.setup.initialize()
-    
+
     if app.qApp.isSessionRestored():
         # Restore session, we are started by the session manager
         session.restoreSession(app.qApp.sessionKey())
         return
 
-    # load specified session?
-    doc = None
-    if args.session and args.session != "-":
-        doc = sessions.loadSession(args.session)
-    
     # Just create one MainWindow
     win = mainwindow.MainWindow()
     win.show()
     win.activateWindow()
-    
+    # make sure all dock tools are initialized and resized
+    app.qApp.processEvents()
+
+    # load specified session?
+    doc = None
+    if args.session and args.session != "-":
+        doc = sessions.loadSession(args.session)
+
     # load documents given as arguments
     import document
     for u in urls:
         doc = win.openUrl(u, args.encoding, ignore_errors=True)
         if not doc:
-            doc = document.Document(u, args.encoding)
-    
+            doc = document.EditorDocument(u, args.encoding)
+
     # were documents loaded?
     if not doc:
         if app.documents:
@@ -262,12 +232,12 @@ def main():
         elif not args.session:
             # no docs, load default session
             doc = sessions.loadDefaultSession()
-    
+
     if doc:
         win.setCurrentDocument(doc)
     else:
         win.cleanStart()
-    
+
     if urls and args.line is not None:
         # set the last loaded document active and apply navigation if requested
         pos = doc.findBlockByNumber(args.line - 1).position() + args.column
@@ -275,5 +245,3 @@ def main():
         cursor.setPosition(pos)
         win.currentView().setTextCursor(cursor)
         win.currentView().centerCursor()
-
-
