@@ -28,6 +28,7 @@ from PyQt5.QtCore import QSettings, QUrl
 
 import app
 import plugin
+import document
 
 
 __all__ = ["info", "define"]
@@ -37,17 +38,17 @@ __all__ = ["info", "define"]
 _defaults = {}
 
 
-def info(document):
+def info(doc):
     """Returns a MetaInfo object for the Document."""
-    return MetaInfo.instance(document)
+    return MetaInfo.instance(doc)
 
 
 def define(name, default, readfunc=None):
     """Define a variable and its default value to be stored in the metainfo.
-    
+
     Should be defined before it is requested or set.
     If readfunc is not given it defaults to a suitable function for bool or int types.
-    
+
     """
     if readfunc is None:
         if isinstance(default, bool):
@@ -60,7 +61,7 @@ def define(name, default, readfunc=None):
         else:
             readfunc = lambda v: v
     _defaults[name] = [default, readfunc]
-    
+
     # read this value for already loaded metainfo items
     for minfo in MetaInfo.instances():
         minfo.loadValue(name)
@@ -68,23 +69,24 @@ def define(name, default, readfunc=None):
 
 class MetaInfo(plugin.DocumentPlugin):
     """Stores meta-information for a Document."""
-    def __init__(self, document):
+    def __init__(self, doc):
         self.load()
-        document.loaded.connect(self.load, -999) # before all others
-        document.closed.connect(self.save,  999) # after all others
-        
+        if doc.__class__ == document.EditorDocument:
+            doc.loaded.connect(self.load, -999) # before all others
+            doc.closed.connect(self.save,  999) # after all others
+
     def settingsGroup(self):
         url = self.document().url()
         if not url.isEmpty():
             s = app.settings('metainfo')
             s.beginGroup(url.toString().replace('\\', '_').replace('/', '_'))
             return s
-        
+
     def load(self):
         s = self.settingsGroup()
         for name in _defaults:
             self.loadValue(name, s)
-        
+
     def loadValue(self, name, settings=None):
         s = settings or self.settingsGroup()
         default, readfunc = _defaults[name]
@@ -100,7 +102,7 @@ class MetaInfo(plugin.DocumentPlugin):
             for name in _defaults:
                 value = self.__dict__[name]
                 s.remove(name) if value == _defaults[name][0] else s.setValue(name, value)
-            
+
 
 @app.aboutToQuit.connect
 def prune():
@@ -110,4 +112,3 @@ def prune():
     for key in s.childGroups():
         if s.value(key + "/time", 0.0, float) < month_ago:
             s.remove(key)
-
