@@ -28,7 +28,7 @@ import os
 
 import app
 import documentinfo
-import jobmanager
+import job.manager
 import plugin
 import util
 
@@ -41,7 +41,7 @@ def results(document):
 @app.jobStarted.connect
 def _init_basenames(document, job):
     results(document).saveDocumentInfo(job.start_time())
-    
+
 
 
 class Results(plugin.DocumentPlugin):
@@ -51,20 +51,20 @@ class Results(plugin.DocumentPlugin):
         self._basenames = None
         self._start_time = 0.0
         document.saved.connect(self.forgetDocumentInfo)
-        
+
     def saveDocumentInfo(self, start_time):
         """Takes over some vital information from a DocumentInfo instance.
-        
+
         This method is called as soon as a job is started.
-        
+
         The file a job is run on and the basenames expected to be created are saved.
         When the user saves a Document after a Job has run, this information is 'forgotten' again.
-        
+
         Otherwise the results of a Job would not be seen if the user starts a Job and then
         saves the Document while the job is still running.  The Job uses the scratcharea if the
         document was modified but saving it would result in DocumentInfo.jobinfo()[0] pointing
         to the real document instead.
-        
+
         """
         info = documentinfo.info(self.document())
         self._start_time = start_time
@@ -73,15 +73,15 @@ class Results(plugin.DocumentPlugin):
 
     def forgetDocumentInfo(self):
         """Called when the user saves a Document.
-        
+
         'Forgets' the basenames and job filename if set, but only if no job is currently running.
-        
+
         """
-        if not jobmanager.is_running(self.document()):
+        if not job.manager.is_running(self.document()):
             self._start_time = 0.0
             self._jobfile = None
             self._basenames = None
-            
+
     def jobfile(self):
         """Returns the file that is currently being, or will be, engraved."""
         if self._jobfile is None:
@@ -96,13 +96,13 @@ class Results(plugin.DocumentPlugin):
 
     def files(self, extension = '*', newer = True):
         """Returns a list of existing files matching our basenames and the given extension.
-        
+
         First the files basename + extension are returned,
         then basename + '-[0-9]+' + extension,
         then basename + '-.+' + extension.
-        
+
         If newer is True (the default), only files that are newer than the jobfile() are returned.
-        
+
         """
         jobfile = self.jobfile()
         if jobfile:
@@ -114,13 +114,13 @@ class Results(plugin.DocumentPlugin):
                     pass
             return list(files)
         return []
-    
+
     def files_lastjob(self, extension = '*'):
         """Like files(), but only returns files that were created by the last job.
-        
+
         If no job has yet run on the document, returns the same files as the
         files_uptodate() method.
-        
+
         """
         if self._start_time:
             files = util.files(self.basenames(), extension)
@@ -131,12 +131,12 @@ class Results(plugin.DocumentPlugin):
             return files
         else:
             return self.files(extension)
-    
+
     def is_newer(self, filename):
         """Return True if the given (generated) file is newer than the jobfile().
-        
+
         Also return True if the mtime of one of the files could not be read.
-        
+
         """
         jobfile = self.jobfile()
         if jobfile:
@@ -145,15 +145,13 @@ class Results(plugin.DocumentPlugin):
             except (OSError, IOError):
                 pass
         return True
-        
+
     def currentDirectory(self):
         """Returns the directory the document resides in.
-        
+
         Returns the temporary directory if that was used last.
-        
+
         """
         directory = os.path.dirname(self.jobfile())
         if os.path.isdir(directory):
             return directory
-
-
