@@ -31,23 +31,23 @@ from PyQt5.QtWidgets import (
 
 class ColorButton(QPushButton):
     """A PushButton displaying a color.
-    
+
     When clicked, opens a color dialog to change the color.
-    
+
     """
     colorChanged = pyqtSignal(QColor)
-    
+
     def __init__(self, parent=None):
         super(ColorButton, self).__init__(parent)
-        
+
         self.setFixedSize(self.sizeHint())
         self._color = QColor()
         self.clicked.connect(self.openDialog)
-    
+
     def color(self):
         """Returns the currently set color."""
         return self._color
-    
+
     def setColor(self, color):
         """Sets the current color. Maybe QColor() to indicate 'unset'."""
         if self._color != color:
@@ -58,7 +58,7 @@ class ColorButton(QPushButton):
     def clear(self):
         """Unsets the current color (setting it to QColor())."""
         self.setColor(QColor())
-        
+
     def openDialog(self):
         """Called when clicked, opens a dialog to change the color."""
         color = self._color if self._color.isValid() else QColor(Qt.white)
@@ -82,5 +82,7 @@ class ColorButton(QPushButton):
             dy = style.pixelMetric(QStyle.PM_ButtonShiftVertical, opt, self)
             r.translate(dx, dy)
         p = QPainter(self)
+        if not self.isEnabled():
+            p.setOpacity(0.5)
         qDrawShadeRect(p, r, self.palette(), True, 1, 0, self._color)
 
