@@ -1,6 +1,6 @@
 # This file is part of the Frescobaldi project, http://www.frescobaldi.org/
 #
-# Copyright (c) 2012 - 2014 by Wilbert Berendsen
+# Copyright (c) 2012 - 2017 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -41,12 +41,12 @@ information.
 
 """
 
-from __future__ import unicode_literals
 
 import collections
 
-from PyQt4.QtCore import QEvent, QObject, QPoint, QRect, QSize, Qt, QTimer
-from PyQt4.QtGui import QPainter, QPalette, QWidget
+from PyQt5.QtCore import QEvent, QObject, QPoint, QRect, QSize, Qt, QTimer
+from PyQt5.QtGui import QPainter, QPalette
+from PyQt5.QtWidgets import QWidget
 
 import cursortools
 
@@ -62,10 +62,10 @@ Level = collections.namedtuple('Level', 'stop start')
 
 class LinePainter(QObject):
     """Paints a line below a block if the next block is invisible.
-    
+
     Install this as an event filter on the viewport() of a textedit,
     it then intercepts the paint event.
-    
+
     """
     def eventFilter(self, obj, ev):
         if ev.type() == QEvent.Paint:
@@ -74,10 +74,10 @@ class LinePainter(QObject):
 
     def paintEvent(self, obj, ev):
         """Paint a line below a block is the next block is invisible.
-        
+
         Normally it calls the paintEvent of the edit, then paints its
         own stuff and returns True.
-        
+
         """
         edit = obj.parent()
         edit.paintEvent(ev)
@@ -96,58 +96,58 @@ class LinePainter(QObject):
 
 class Folder(QObject):
     """Manages the folding of a QTextDocument.
-    
+
     You should inherit from this class to provide folding events.
     It is enough to implement the fold_events() method.
-    
+
     By default, simple caching is used to store the nesting depth every
     20 lines. This makes the depth() method faster, which would otherwise count
     the fold_events() for every block from the beginning of the document.
-    
+
     The depth() caching expects that the fold_events that a text block
     generates do not depend on the contents of a text block later in the
     document.
-    
+
     If your fold_events() method generates events for a text block that depend
     on a later block, you should disable caching by setting the
     cache_depth_lines instance (or class) attribute to zero.
-    
+
     """
     # cache depth() for every n lines (0=disable)
     cache_depth_lines = 20
-    
+
     def __init__(self, doc):
         QObject.__init__(self, doc)
         self._depth_cache = []      # cache result of depth()
         self._all_visible = None    # True when all are certainly visible
         doc.contentsChange.connect(self.slot_contents_change)
         self._timer = QTimer(singleShot=True, timeout=self.check_consistency)
-    
+
     @classmethod
     def find(cls, doc):
         for c in doc.children():
             if type(c) is cls:
                 return c
-    
+
     @classmethod
     def get(cls, doc):
         return cls.find(doc) or cls(doc)
-    
+
     def slot_contents_change(self, position, removed, added):
         """Called when the document changes.
-        
+
         Provides limited support for unhiding regions when the user types
         text in it, and deletes the depth() cache for lines from position.
-        
+
         """
         block = self.document().findBlock(position)
         if self.cache_depth_lines:
             chunk = block.blockNumber() // self.cache_depth_lines
             del self._depth_cache[chunk:]
-        
+
         if self._all_visible:
             return
-        
+
         if not block.isVisible():
             self.ensure_visible(block)
         else:
@@ -165,27 +165,28 @@ class Folder(QObject):
                             if self.fold_level(n).start:
                                 continue
                     n = n.next()
-                self.document().markContentsDirty(block.next().position(), n.position())
+                start = block.next().position()
+                self.document().markContentsDirty(start, n.position() - start)
         self._timer.start(250 + self.document().blockCount())
-    
+
     def invalidate_depth_cache(self, block):
         """Makes sure the depth is recomputed from the specified block."""
         if self.cache_depth_lines:
             chunk = block.blockNumber() // self.cache_depth_lines
             del self._depth_cache[chunk:]
-    
+
     def check_consistency(self):
         """Called some time after the last document change.
-        
+
         Walk through the whole document, unfolding folded lines that
         - are in the toplevel
         - are in regions that have visible lines
         - are in regions that have visible sub-regions
-        
+
         """
-        show_blocks = set()
+        show_blocks = []
         self._all_visible = True    # for now at least ...
-        
+
         def blocks_gen():
             """Yield depth (before block), block and fold_level per block."""
             depth = 0
@@ -195,17 +196,17 @@ class Folder(QObject):
                 depth += sum(l)
 
         blocks = blocks_gen()
-        
+
         def check_region(start_block, start_depth):
             """Check a region from a starting block for visible lines.
-            
+
             Return a four-tuple (must_show, depth, block, level).
             must_show is True if the region contains visible lines (in that case
                 the invisible lines are already made visible)
             depth, block, and level are the result of the last block_gen yield.
             If level.start is True, a new region starts on the same line the
             former one ends.
-            
+
             """
             must_show = False
             invisible_blocks = []
@@ -217,7 +218,7 @@ class Folder(QObject):
                     if block.isVisible() and not level.start:
                         must_show = True
                     if must_show:
-                        show_blocks.update(invisible_blocks)
+                        show_blocks.extend(invisible_blocks)
                     elif invisible_blocks:
                         self._all_visible = False
                     return must_show, depth, block, level
@@ -229,45 +230,46 @@ class Folder(QObject):
                         must_show = True
             # happens if region is not closed
             return must_show, 0, None, Level(0, 0)
-        
+
         # toplevel
         for depth, block, level in blocks:
-            block.isVisible() or show_blocks.add(block)
+            block.isVisible() or show_blocks.append(block)
             while level.start:
                 must_show, depth, block, level = check_region(block, depth + sum(level))
-        
+
         if show_blocks:
             for block in show_blocks:
                 block.setVisible(True)
-            self.document().markContentsDirty(
-                min(show_blocks).position(), max(show_blocks).position())
-    
+            start = show_blocks[0].position()
+            end = show_blocks[-1].position() + show_blocks[-1].length()
+            self.document().markContentsDirty(start, end - start)
+
     def document(self):
         """Return our document."""
         return self.parent()
-        
+
     def fold_events(self, block):
         """Return an iterable of fold events for the block.
-        
+
         An event is simply an integer constant START or END.
         The default implementation considers '{' as a start and '}' as end.
-        
+
         """
         for c in block.text():
             if c == '{':
                 yield START
             elif c == '}':
                 yield STOP
-    
+
     def fold_level(self, block):
         """Returns a named two-tuple Level(stop, start) about the block.
-        
+
         stop is the number (negative!) of fold-levels that end in that block,
         start is the number of fold-levels that start in that block.
-        
+
         This methods uses fold_events() to get the information, it discards
         folding regions that start and stop on the same text line.
-        
+
         """
         start, stop = 0, 0
         for e in self.fold_events(block):
@@ -278,14 +280,14 @@ class Folder(QObject):
             else:
                 stop -= 1
         return Level(stop, start)
-        
+
     def depth(self, block):
         """Return the number of active regions at the start of this block.
-        
+
         The default implementation simply counts all the fold_events from
         the beginning of the document, using caching if the cache_depth_lines
         instance attribute is set to a value > 0.
-        
+
         """
         depth = 0
         last = block.document().firstBlock()
@@ -310,18 +312,18 @@ class Folder(QObject):
             depth += sum(self.fold_events(last))
             last = last.next()
         return depth
-        
+
     def region(self, block, depth=0):
         """Return as Region (start, end) the region of the specified block.
-        
+
         start is the block the region starts, end the block the region ends.
-        When collapsing the block, dont hide the last block if it starts a new
+        When collapsing the block, don't hide the last block if it starts a new
         fold region.
-        
+
         The depth argument specifies how deep a region may be nested.
         The default value 0 searches the first containing region, 1 tries to
         find one more above that, etc. Use -1 to get the top-most region.
-        
+
         """
         start = None
         start_depth = 0
@@ -346,14 +348,14 @@ class Folder(QObject):
                 count += sum(l)
             if end:
                 return Region(start, end)
-        
+
     def fold(self, block, depth=0):
         """Fold the region the block is in.
-        
+
         The depth argument specifies how deep a region may be nested.
         The default value 0 searches the first containing region, 1 tries to
         find one more above that, etc. Use -1 to get the top-most region.
-        
+
         """
         r = self.region(block, depth)
         if not r:
@@ -365,23 +367,24 @@ class Folder(QObject):
         for block in cursortools.forwards(r.start.next(), end):
             block.setVisible(False)
         self.mark(r.start, True)
-        self.document().markContentsDirty(r.start.next().position(), end.position())
+        start = r.start.next().position()
+        self.document().markContentsDirty(start, end.position() - start)
         self._all_visible = False
 
     def unfold(self, block, depth=0, full=False):
         """Unfolds the region the block is in.
-        
+
         (Most times the block will be the first block of the region.)
         If multiple regions start at the same starting block, they will unfold
         all.
-        
+
         The depth argument specifies how deep a region may be nested.
         The default value 0 searches the first containing region, 1 tries to
         find one more above that, etc. Use -1 to get the top-most region.
-        
+
         If full is False (the default) sub-regions that were collapsed remain
         collapsed (provided that the mark() method is implemented).
-        
+
         """
         r = self.region(block, depth)
         if not r:
@@ -403,8 +406,9 @@ class Folder(QObject):
                             count += sum(l)
             block.setVisible(True)
         self.mark(r.start, False)
-        self.document().markContentsDirty(r.start.position(), r.end.position())
-        
+        start = r.start.position()
+        self.document().markContentsDirty(start, r.end.position() - start)
+
     def fold_toplevel(self):
         """Folds all toplevel regions, without touching inner regions."""
         for block in cursortools.all_blocks(self.document()):
@@ -412,7 +416,7 @@ class Folder(QObject):
                 continue
             elif self.fold_level(block).start:
                 self.fold(block)
-    
+
     def fold_all(self):
         """Folds all regions."""
         for block in cursortools.all_blocks(self.document()):
@@ -421,7 +425,7 @@ class Folder(QObject):
                     self.fold(block)
                 else:
                     self.mark(block, True)
-    
+
     def unfold_all(self):
         """Fully unfolds the document."""
         first = None
@@ -434,24 +438,24 @@ class Folder(QObject):
                     first = block
                 last = block
         if first:
-            self.document().markContentsDirty(first.position(), last.position())
+            self.document().markContentsDirty(first.position(), last.position() - first.position())
         # no need to check consistency
         self._all_visible = True
         self._timer.isActive() and self._timer.stop()
-        
+
     def mark(self, block, state=None):
         """This can be used to remember the folded state of a block.
-        
+
         When folding or unfolding a block, this method is called with the first
         block and state True (for fold) or False (for unfold).
-        
+
         When unfolding a region, this method is called for every block without
         state value, and its return value is checked. If the value evaluates to
         True, the sub-region will remain collapsed.
-        
+
         The default implementation does, and returns, nothing.
-        
-        """ 
+
+        """
         pass
 
     def ensure_visible(self, block):
@@ -465,15 +469,15 @@ class Folder(QObject):
 
 
 class FoldingArea(QWidget):
-    
+
     Folder = Folder
-    
+
     class Painter(object):
         """Used for one paint event, draws the folding area per-block."""
         def __init__(self, widget):
             self.w = widget
             self.p = QPainter(widget)
-        
+
         def draw(self, rect, indicator, depth, new_depth):
             p = self.p
             if depth:
@@ -492,13 +496,13 @@ class FoldingArea(QWidget):
                 p.drawLine(QPoint(x-2, y), QPoint(x+2, y))
                 if indicator == OPEN:
                     p.drawLine(QPoint(x, y-2), QPoint(x, y+2))
-    
+
     def __init__(self, textedit=None):
         super(FoldingArea, self).__init__(textedit)
         self._textedit = None
         self.setAutoFillBackground(True)
         self.setTextEdit(textedit)
-    
+
     def setTextEdit(self, edit):
         """Set a QPlainTextEdit instance to show folding indicators for, or None."""
         if self._textedit:
@@ -509,18 +513,18 @@ class FoldingArea(QWidget):
             edit.updateRequest.connect(self.slotUpdateRequest)
             edit.cursorPositionChanged.connect(self.slotCursorPositionChanged)
         self.update()
-        
+
     def textEdit(self):
         """Return our QPlainTextEdit."""
         return self._textedit
-        
+
     def sizeHint(self):
         return QSize(11, 50)
-    
+
     def folder(self):
         """Return the Folder instance for our document."""
         return self.Folder.get(self.textEdit().document())
-        
+
     def slotUpdateRequest(self, rect, dy):
         if dy:
             self.scroll(0, dy)
@@ -533,7 +537,7 @@ class FoldingArea(QWidget):
         if not block.isVisible():
             self.folder().ensure_visible(block)
             self.textEdit().ensureCursorVisible()
-    
+
     def paintEvent(self, ev):
         edit = self._textedit
         if not edit:
@@ -569,7 +573,7 @@ class FoldingArea(QWidget):
                     painter.draw(rect, indicator, depth, depth + count)
             depth += count
             block = next_block
-    
+
     def mousePressEvent(self, ev):
         if ev.buttons() == Qt.LeftButton:
             block = self.textEdit().cursorForPosition(QPoint(0, ev.y())).block()
@@ -583,9 +587,9 @@ class FoldingArea(QWidget):
 
 def visible_blocks(edit, rect=None):
     """Yield the visible blocks in the specified rectangle.
-    
+
     If no rectangle is given, the edit's viewport() is used.
-    
+
     """
     if rect is None:
         rect = edit.viewport().rect()
